#import <eoaccess/eoaccess.h>
#import <WebObjects/WOApplication.h>

@class WOComponentController;

////////////////////////////////////////////////////////////////////////////
//
// This is data that really belongs to each entity.
// Don't put it in the model, because that's too tough to manage
// (Have to keep all of the different .eomodel's in sync.)
//
// Put this somewhere outside the model, but easily accessible.
// (Besides, that's how EOF 2.0 works...)
//
////////////////////////////////////////////////////////////////////////////

	//
	// The .plist extension to the .eomodel file is a
	// lightweight mechanism to tailor functionality in
	// the CRUD Objects Layer without writing code.
	//
	// The data defined here has no associated logic.
	//
	// To add logic to this data, one needs to implement
	// this on the entity or object instance.
	//
@interface CRUDEOModelData : NSObject
{
	NSDictionary *modelData;
}

- initWithModelName:(NSString*)filename;
	// Set the name of .plist file.
	// This should be the <filename> from <filename>.eomodel

- (NSDictionary*)modelData;
	// This is the .plist.
	// The .plist is the extension to the .eomodel data
	// for both entities and attributes.

- (id)
     _objectForName:(NSString*)aName
  onDictionaryNamed:(NSString*)dictionaryName
     forEntityNamed:(NSString*)entityName;

- (NSString*)
           _external:(BOOL)external
      valueForObject:(id)aValue
         forProperty:(NSString*)property
       onEntityNamed:(NSString*)entityName;

- (void) dealloc;
@end

	//
	// Override the default CRUDLayer without logic.
	// Basically a lightweight mechanism to override
	// the values without having to code.
	//
@interface CRUDEOModelData (CRUDLayer)
- (WOComponentController *)pageWithName:(NSString *)aName forEntityNamed:(NSString*)entityName;
	// Override the default CRUD page.
	// (e.g. return a tailored page for something like
	// the CRUDEditorPage for the specified entity.)

- (NSArray *)pageProperties:aName forEntity:(EOEntity *)anEntity;
	// Override the default properties used for any of the
	// page functions.
	// (e.g. sortProperties, editableProperties, etc.)

- (BOOL)objectIsEditable:(id)anObject;
        // Ascertain whether or not the specified
        // object should be editable.
	// Default is YES.

- (NSNumber*)widthOfProperty:(NSString*)property onEntity:(EOEntity*)entity;
- (NSNumber*)heightOfProperty:(NSString*)property onEntity:(EOEntity*)entity;
@end

	//
	// Allow attributes to use a convert internal (Database)
	// values to an external (UserInterface) value.
	// This is really handy when an attribute should only be
	// set to an enumerated list of values, and the database
	// values are not entirely meaningful to the user.
	// (e.g. attribute 'status' has db values of {0,1,2}
	// which should correspond to values in the interface of
	// {"Open","InProgress","Closed"}.)
	//
@interface CRUDEOModelData (DataConversion)
- (NSArray*)
    possibleValuesForProperty:(NSString*)property
                onEntityNamed:(NSString*)entityName;
- (NSString*)
    externalValueForObject:(id)internalKey
	       forProperty:(NSString*)property
             onEntityNamed:(NSString*)entityName;
- (NSString*)
    internalValueForObject:(id)externalValue
	       forProperty:(NSString*)property
             onEntityNamed:(NSString*)entityName;
@end

