#import "CRUDEOModelData.h"
#import "NSDictionary+MKConvenience.h"

@implementation CRUDEOModelData
	
	// TODO: This should be implemented using sharedInstance?
- initWithModelName:(NSString*)filename
{
	id type = @"plist";
	id path;

	[super init];

	modelData = nil;
	
	path = [WOApp pathForResource:filename ofType:type];
	if (!path) {
		NSLog(@"Optional pathForResource:%@.%@ not found, ignoring...",
			filename, type);
 		return self;	// not providing this file is not an error...
	}
	modelData = [[NSDictionary alloc] initWithContentsOfFile:path];
	if (!modelData) {
		NSLog(@"ERROR: CRUDEOModelData: failed to create modelData from pathForResource:%@.%@", filename, type);
		return nil;
	}
	return self;
}

- (NSDictionary*)modelData { return modelData; }

- (id)
      _objectForName:(NSString*)aName
  onDictionaryNamed:(NSString*)dictionaryName
     forEntityNamed:(NSString*)entityName
{
	NSDictionary *entityDict;
	NSDictionary *namedDict;
	
	entityDict = [modelData objectForKey:entityName];
	if (!entityDict)
		return nil;
	namedDict = [entityDict objectForKey:dictionaryName];
	if (!namedDict)
		return nil;
	return [namedDict objectForKey:aName];
}

- (NSString*)
            _external:(BOOL)external
      valueForObject:(id)aValue
         forProperty:(NSString*)property
       onEntityNamed:(NSString*)entityName
{
	NSDictionary *propDict;
	NSArray *dataConversionArray;
	NSEnumerator *enu;
	NSDictionary *dataConversionDict;
	id anObject;

	propDict = [self
			_objectForName:property
			onDictionaryNamed:@"properties"
			forEntityNamed:entityName];
	dataConversionArray = [propDict objectForKey:@"dataConversion"];
	if (!dataConversionArray)
		return nil;

	aValue = [aValue description];		
	enu = [dataConversionArray objectEnumerator];
	while (dataConversionDict = [enu nextObject]) {
	    if (external)
		anObject = [dataConversionDict objectForKey:aValue];
	    else
		anObject = [dataConversionDict keyForObject:aValue];
		if (anObject)
			return [anObject description];
	}
	return nil;
}


- (void) dealloc { [modelData release]; }
@end

@implementation CRUDEOModelData (CRUDLayer)
- (WOComponentController *)
     pageWithName:(NSString *)aName
   forEntityNamed:(NSString*)entityName
{
	NSString *pageName;	
	pageName = [self
			_objectForName:aName
			onDictionaryNamed:@"pages"
			forEntityNamed:entityName];
	if (!pageName)
		return nil;
	return [WOApp pageWithName:pageName];
}

- (NSArray *)
  pageProperties:aName
       forEntity:(EOEntity *)anEntity
{
	NSArray *names;	
	names = [self
			_objectForName:aName
			onDictionaryNamed:@"pageProperties"
			forEntityNamed:[anEntity name]];
	return names;
}

- (BOOL)objectIsEditable:(id)anObject
{
    NSString *editableString;
    NSDictionary *entityDict;

    entityDict = [modelData objectForKey:[[anObject entity] name]];
    if (!entityDict)
            return YES;
    editableString = [entityDict objectForKey:@"isEditable"];
    if (!editableString)
            return YES;
    if ([editableString isEqualToString:@"NO"])
        	return NO;
    return YES;
}

- (NSNumber*)widthOfProperty:(NSString*)property onEntity:(EOEntity*)entity
{
	NSDictionary *propertyDict;
	NSString *aValue;

	propertyDict = [self
			_objectForName:property
			onDictionaryNamed:@"properties"
			forEntityNamed:[entity name]];
	aValue = [propertyDict objectForKey:@"width"];
	if (!aValue)
		return nil;
	return [NSNumber numberWithInt:[aValue intValue]];
}

- (NSNumber*)heightOfProperty:(NSString*)property onEntity:(EOEntity*)entity
{
	NSDictionary *propertyDict;
	NSString *aValue;

	propertyDict = [self
			_objectForName:property
			onDictionaryNamed:@"properties"
			forEntityNamed:[entity name]];
	aValue = [propertyDict objectForKey:@"height"];
	if (!aValue)
		return nil;
	return [NSNumber numberWithInt:[aValue intValue]];
}
@end


@implementation CRUDEOModelData (DataConversion)
- (NSArray*)
    possibleValuesForProperty:(NSString*)property
                onEntityNamed:(NSString*)entityName
{
	NSDictionary *propertyDict;
	NSArray *dataConversionArray;
	NSEnumerator *enu;
	NSDictionary *dataConversionDict;
	NSMutableArray *possibleValues;

	propertyDict = [self
			_objectForName:property
			onDictionaryNamed:@"properties"
			forEntityNamed:entityName];
	dataConversionArray = [propertyDict objectForKey:@"dataConversion"];
	if (!dataConversionArray)
		return nil;
	
	possibleValues = [NSMutableArray array];
	enu = [dataConversionArray objectEnumerator];
	while (dataConversionDict = [enu nextObject]) {
		// blech...
		// This is a result of storing the dataConversion
		// stuff as an array of dictionaries, each with
		// one entry...
		// (is there a better way to store this stuff?)
		[possibleValues addObject:
			[[dataConversionDict allValues] objectAtIndex:0]];
	}
	return possibleValues;
}

- (NSString*)
    externalValueForObject:(id)internalKey
	       forProperty:(NSString*)property
             onEntityNamed:(NSString*)entityName
{
	   return [self	_external:YES	
			valueForObject:internalKey
			forProperty:property
			onEntityNamed:entityName];
}
	     
- (NSString*)
    internalValueForObject:(id)externalValue
	       forProperty:(NSString*)property
             onEntityNamed:(NSString*)entityName
{
	   return [self	_external:NO
			valueForObject:externalValue
			forProperty:property
			onEntityNamed:entityName];
}

@end

