#import <WebObjects/WOApplication.h>
#import <eoaccess/eoaccess.h>
#import "CRUDEOModelData.h"

@class WOComponentController;

@interface CRUDLayer : NSObject
{
	NSMutableDictionary *dataSources;
	EOModel *model;
	CRUDEOModelData *modelData;
}

- initWithModelName:(NSString*)modelName;
	// Sole init method for the CRUDLayer.
	// [Currently, this is being called once per application,
	// but wondering if this should be session based, similar to 
	// the UserSpace stuff.]
	
- (EOModel*)model;
- (CRUDEOModelData*)modelData;
@end

@interface CRUDLayer (DataSources)
- (EODatabaseDataSource*)dataSourceForEntityNamed:(NSString*)entityName;
@end

@interface CRUDLayer (WOAppGoodies)
- (WOComponentController *)pageWithName:(NSString *)aName forEntity:(EOEntity*)anEntity;
- (WOComponentController *)pageWithName:(NSString *)aName forObject:(id)anObject;
@end

@interface CRUDLayer (EOAccessLayerAdditions)
- (NSNumber*)widthOfProperty:(NSString*)property onEntity:(EOEntity*)entity;
- (NSNumber*)heightOfProperty:(NSString*)property onEntity:(EOEntity*)entity;
@end

@interface CRUDLayer (PropertyHandling)
- (NSArray*)qualifierBuilderPropertiesForEntity:(EOEntity *)anEntity;
- (NSArray*)sortablePropertiesForEntity:(EOEntity *)anEntity;
- (NSArray*)pickListPropertiesForEntity:(EOEntity *)anEntity;
	// In the context for the methods above, 
	// only the entity is available.

- (NSArray*)visiblePropertiesForObject:(id)anObject;
- (NSArray*)editablePropertiesForObject:(id)anObject;
	// ...after an EO has been selected (in the pickList)
	// the instance should be queried.
	// Therefore, the interface changes slightly
	// to support calls to the EO instance.

- (BOOL)objectIsEditable:(id)anObject;
	// Ascertain whether or not the specified
	// object should be editable.
	// Default is YES.

	// uggg...
- (NSArray*)sortColumnNamesForProperties:(NSArray*)properties;
@end


// #define CRUD_DEBUG

#ifdef CRUD_DEBUG
@interface CRUDLayer (DebuggingDelegateMethods)
- (EODelegateResponse)adaptorChannel:channel willEvaluateExpression:(NSMutableString *)anExpression;
- (BOOL)adaptor:(EOAdaptor *)adaptor willReportError:(NSString *)error;
@end
#endif
