#import "foundation.h"
#import <eoaccess/eoaccess.h>
#import <WebObjects/WOApplication.h>

@protocol CRUDObjectProtocol

+ (WOComponentController *)pageWithName:(NSString *)aName;
- (WOComponentController *)pageWithName:(NSString *)aName;
	// The Class has responsibility for supplying
	// the .wo pages to interact with the Class/Object.
	// This method will only be meaningful in a subclass which
	// overrides this method.
	// [Typically one asks WOApp for the pageWithName, but
	// this now depends upon the class the user is interacting with.]
	// FYI: Haven't made it far enough yet to determine if both class
	// and instance methods are really needed.  The instance is, I 
	// believe, because I might like to determine the Editor, or
	// Status page based on the values of the instance.
	// The Class method is needed because we don't always have
	// an instance of this class to work with (i.e. in the
	// QualifierBuilderPage.wo)

+ (NSArray*)qualifierBuilderProperties;
	// The array of property names that should be used in building
	// a QualiferBuilder for the specified object.

+ (NSArray*)sortableProperties;
	// The array of property names that can be used in creating
	// an EOAttributeOrdering for the objects dataSource.

+ (NSArray*)pickListProperties;
	// The array of property names that should be used in creating a
	// PickList for the specified object.

- (NSArray*)visibleProperties;
	// The array of property names that are visible.
	// By default, all properties are visible.

- (NSArray*)editableProperties;
	// The array of property names that are editable.
	// By default, all properties are editable.
	
- (NSArray*)actions;
	// If the object supports "actions" (an action to perform on the
	// objects) the array of selector names are returned here.
	// [This hook is here to implement some Entity-Life-Cycle or
	// WorkFlow behvior with Entities.  Some entities contain
	// "static" data, or lookup data, like "locations".  Buildings
	// don't really have a corresponding status, like Open, New,
	// Closed, etc.  (Well, for most apps/data, anyway.)
	// Entities like "ChangeRequest", "WorkOrder", "InspectionRoutine"
	// might all have a "state".  "In Progress", "Done", etc.
	// The "actions" supplied by this instance method are the
	// actions a user can take against the instance.


+ (NSNumber*)widthOfProperty:(NSString*)property;
+ (NSNumber*)heightOfProperty:(NSString*)property;

- (NSString*)hasValidData;
	// Should be called before and EO saves itself to the dataSource
	// (This is the EO's chance to turn back bogus data, and for integrity)
	// If there is an error a string describing the error is returned.
	// If there is no error nil is returned.
	
- (NSString*)initializeSelf;
	// Should only be called once on a CRUD object
	// (i.e. in conjunction with [dataSource createObject])
	// This method should set the newly gen'ed EO to something reasonable
	// (i.e. this is the EO's chance to set things like "dateCreated" or 
	// set other default values.
	// If there is an error a string describing the error is returned.
	// If there is no error nil is returned.
@end

