#import "EOEntity+CRUDLayerExtensions.h"

@implementation EOEntity (CRUDLayerExtensions)

- (Class)classNameClass
{
	return NSClassFromString([NSString stringWithCString:[self className]]);
}

- (NSArray*)attributeNames
{
	NSEnumerator *enu;
	NSMutableArray *names;
	EOAttribute *anAttrib;
	
	names = [NSMutableArray array];
	enu = [[self attributes] objectEnumerator];
	while (anAttrib = [enu nextObject]) {	
		[names addObject:[anAttrib name]];
	}
	return names;
}

- (NSArray*)visibleAttributes
{
	NSEnumerator *enu;
	NSMutableArray *attribs;
	EOAttribute *anAttrib;
	EORelationship *aRelationship;
	
	attribs = [[self attributes] mutableCopy];
	[attribs autorelease];

	//
	//  Everything, but the PrimaryKey
	//  (This should really be configurable...TODO)
	//
	[attribs removeObjectsInArray:[self primaryKeyAttributes]];

	//
	//  ...and the internal portion of Flattened Attributes.
	//
	enu = [[self relationships] objectEnumerator];
	while (aRelationship = [enu nextObject]) {
		anAttrib = [[aRelationship sourceAttributes] objectAtIndex:0];
		[attribs removeObject:anAttrib];
	}
	return attribs;
}

- (NSArray*)visibleAttributeNames
{
	NSEnumerator *enu;
	NSMutableArray *names;
	EOAttribute *anAttrib;
	
	names = [NSMutableArray array];
	enu = [[self visibleAttributes] objectEnumerator];
	while (anAttrib = [enu nextObject]) {	
		[names addObject:[anAttrib name]];
	}
	return names;
}
@end

	//
	// Way over the top on this one...
	// This API (width, height) isn't supported by EOF Access Layer.
	// Any good ideas on how to achieve this????
	// This is a big HACK.  (TODO)
	//
@implementation EOEntity (EOAccessLayerExtensions)
	
#define MAXPROPERTYWIDTH	80
#define DEFAULTPROPERTYWIDTH	20

- (NSNumber*)widthOfProperty:(NSString*)property
{
	int length;
	
	length = [[self lengthOfProperty:property] intValue];
	if (!length)
		length = DEFAULTPROPERTYWIDTH;
	while (length > MAXPROPERTYWIDTH)
		length /= 2;
	return [NSNumber numberWithInt:length];
}

- (NSNumber*)heightOfProperty:(NSString*)property
{
	int length;
	int count = 1;
	
	length = [[self lengthOfProperty:property] intValue];
	while (length > MAXPROPERTYWIDTH) {
		length /= 2;
		count++;
	}
	return [NSNumber numberWithInt:count];
}

- (NSNumber*)lengthOfProperty:(NSString*)property
{
	EOAttribute *attribute;
	NSString *externalType;
	NSString *widthString;
	NSArray *array;
	
	attribute = [self attributeNamed:property];
	externalType = [attribute externalType];
	
	array = [externalType componentsSeparatedByString:@"("];
	if ([array count] != 2)
		// this is dates, integers, etc...
		return [NSNumber numberWithInt:DEFAULTPROPERTYWIDTH];
		
	array = [[array objectAtIndex:1] componentsSeparatedByString:@")"];
	if ([array count] != 2)
		// this is dates, integers, etc...
		return [NSNumber numberWithInt:DEFAULTPROPERTYWIDTH];
	
	widthString = [array objectAtIndex:0];
	
	return [NSNumber numberWithInt:[widthString intValue]];
}

@end




