//////////////////////////////////////////////////////////////////////////
//
//	ComponentEditor.app
//	FileName: AppController.m
//
//		Written by zander@cnext.crec.mipt.ru (Aleksey Sudakov)
// 	Feb. 1,1996
//		Last Modified Feb. 1,1996
//

#import "AppController.h"
#import "ComponentEditor.h"

@implementation AppController
- init
{
	[super init];
	first = YES;
	return self;
}


- showPrefs:sender
{
//	if(!prefController)
//		[NXApp loadNibSection:"Preferences.nib" owner:self];
//	[[prefController window] makeKeyAndOrderFront:sender];
	return self;
}

- new:sender
{
	id win;

	// get a new doc window.
	if(![NXApp loadNibSection:"EditorWindow.nib" owner:self withNames:NO]) 
		return nil;
	[editor init];
	win = [editor window];
	if( win ){
		NXRect frame;

		[win getFrame: &frame];
		NX_X(&frame) += 10;
		NX_Y(&frame) -= 10;
		[win placeWindowAndDisplay: &frame];
		[win setTitleAsFilename:""];
 		[win makeKeyAndOrderFront:self];
 	}
	return self;
}
- open:sender
{	// use open panel to select a file -- only with .wo extension.
	// only one file may be loaded at a time.
	const char *const *files;
	char *file;
	const char *dir;
	static const char *const ft[2] = {"wo", NULL};
	id openPanel = [OpenPanel new];

	[openPanel allowMultipleFiles:NO];
	if (first) {
		[openPanel runModalForDirectory:[[NXBundle mainBundle] directory]
				file:NULL types:ft];
		first = NO;
	} else  [openPanel runModalForTypes: ft];
	files = [openPanel filenames];
	dir = [openPanel directory];
	if (files)
	{	
		file = malloc(strlen(files[0]) + strlen(dir) + 8);
		strcpy(file, dir);
		strcat(file,"/");
		strcat(file, files[0]);
		strcat(file, "\0");
		[self openFile:file];
	}
	return self;
}

- openFile:(const char *)fileName
{
	id win;

	// get a new doc window.
	if(![NXApp loadNibSection:"EditorWindow.nib" owner:self withNames:NO]) 
		return nil;
	[editor initFromDirectory:fileName];
	win = [editor window];
	if( win ){
		NXRect frame;

		[win getFrame: &frame];
		NX_X(&frame) += 10;
		NX_Y(&frame) -= 10;
		[win placeWindowAndDisplay: &frame];
		[win setTitleAsFilename:fileName];
 		[win makeKeyAndOrderFront:self];
 	}
	return self;
}

- (int)countEditedWindows
{
	id winList;
	int i;
	int count = 0;

	winList = [NXApp windowList];
	for(i=0; i<[winList count]; i++)
		if( [[winList objectAt: i] isDocEdited]) count ++;
	return count;
}

- saveAll:sender
{
	id winList;
	int i;

	winList = [NXApp windowList];
	for(i=0; i<[winList count]; i++){
		id win = [winList objectAt:i];
		id delegate = [win delegate];

	 	if( [delegate isKindOf:[ComponentEditor class]] ){
			[win makeKeyAndOrderFront:nil];
			[delegate save:win];
		}
	}
	return self;
}

- (BOOL)menuActive:menuCell
{
	BOOL shouldBeEnabled;

	shouldBeEnabled = [[[NXApp mainWindow] delegate] 
							isKindOf:[ComponentEditor class]];
	if ( [menuCell isEnabled] != shouldBeEnabled ){
	// is either enabled and shoun't be or visa versa
		[menuCell setEnabled: shouldBeEnabled];
	 	return YES;		// redisplay
	}
	return NO;			// no change
}

- appDidInit:(Application *)sender
{
	BOOL haveOpenedDocument = NO;	// whether we have opened a document
	id docMenu = [componentSubmenuCell target];

	if (NXArgc > 1){
		int i;

		for (i = 1; i < NXArgc; i++)
			haveOpenedDocument = [self openFile:NXArgv[i]] || 
								haveOpenedDocument;
	 }
	[saveMenuCell setUpdateAction:@selector(menuActive:) forMenu:docMenu];
	[saveAsMenuCell setUpdateAction:@selector(menuActive:) forMenu:docMenu];
	[saveAllMenuCell setUpdateAction:@selector(menuActive:) forMenu:docMenu];
	[NXApp setAutoupdate: YES];

	return self;
}

- appWillTerminate:sender
{
 if( [self countEditedWindows] > 0 )
	{
	 int q = NXRunAlertPanel("Quit","There are edited windows.",
					"Review Unsaved", "Quit Anyway","Cancel");
	 if( q == 1)	
		{	//	Review
		 int i;
		 id winList;
	
		 winList = [NXApp windowList];
		 for(i = 0; i< [winList count]; i++)
			{
			 id win = [winList objectAt:i];
			 
			 if( [[win delegate] isKindOf:[ComponentEditor class]] )
				[win performClose:nil];
			}
		 return self;
		}
	if( q == -1)	return nil;		//	Cancel;
	}
 return self;
}

// The next two methods allow the WorkSpace to open a .wo file when
// it is double-clicked.  (Or any file that's cmd-dragged over our icon.)

- (BOOL)appAcceptsAnotherFile:sender {
	return YES; 
}

- (int)app:sender openFile:(const char *)file type:(const char *)type{
	return (int)[self openFile:file];
}
@end
