//////////////////////////////////////////////////////////////////////////
//
//	ComponentEditor.app
//	FileName: ComponentEditor.h
//
//	Written by zander@cnext.crec.mipt.ru (Aleksey Sudakov)
// 	Feb. 2,1996
//		LastModified: March 24, 1996
//

#import "ComponentEditor.h"
#import <appkit/SavePanel.h>

extern errno;

@implementation ComponentEditor

- awakeFromNib
{
	return [self init];
}

- init
{

	dir = [[NSString alloc] init];
	template = [[NSString alloc] init];
	declaration = [[NSString alloc] init];
	script = [[NSString alloc] init];
    	[self _initTexts];

	return self;
}

- (void)_initTexts
{
	id errs = nil;
	NSString *path = [[NSString alloc] init];
	XTDispatchAction *action = [[XTDispatchAction alloc] init];
	
		
	path =[[NSBundle mainBundle] 
			pathForResource:@"KeyBindingFiles"
			ofType:@""];
	if ([path length] > 0)
	{
		const char *keyfiles;
		NSString *fileName;
		NSEnumerator *keyEnum;
		NSString *keyFileString;
		NSArray *keyFileArray = nil;
		
		if (keyfiles = NXGetDefaultValue("ComponentEditor", "KeyFiles"))
		{
			keyFileString = [NSString stringWithCString:keyfiles];
			keyFileArray = [keyFileString componentsSeparatedByString:@":"];
		}
		if ([keyFileArray count] <= 0)
		{
			 // no key files selected, use the following default list
			keyFileArray = [NSArray arrayWithObjects: @"arrow.keys", @"emacs.keys", @"keypad.keys", @"programmer.keys", @"readonly.keys", nil];
		}
		keyEnum = [keyFileArray objectEnumerator];
		while (fileName = [keyEnum nextObject])
		{
			NSString *newPath = [[NSString alloc] init];
			newPath = [newPath stringByAppendingString:path];
			newPath = [newPath stringByAppendingPathComponent:fileName];
			[action loadFromFile:(char *)[newPath cString] estream:errs];
		}
	}
 
    	templateText = [templateScrollView docView];
    	declarationText = [declarationScrollView docView];
    	scriptText = [scriptScrollView docView];

	[templateText setInitialAction:action];
	[declarationText setInitialAction:action];
	[scriptText setInitialAction:action];

	[templateText setDelegate:self]; 
	[declarationText setDelegate:self]; 
	[scriptText setDelegate:self]; 

	[templateText setMonoFont:YES];
	[templateText setFont:[Font userFixedPitchFontOfSize: 0.0 
				    matrix: NX_FLIPPEDMATRIX]];
	[declarationText setMonoFont:YES];
	[declarationText setFont:[Font userFixedPitchFontOfSize: 0.0 
				    matrix: NX_FLIPPEDMATRIX]];
	[scriptText setMonoFont:YES];
	[scriptText setFont:[Font userFixedPitchFontOfSize: 0.0 
				    matrix: NX_FLIPPEDMATRIX]];

	[path release];
}

- (void)_initStyles{
	NSArray* keywords;
	NSDictionary* style;
	id font;		// Should be NSFont* font;
	id color;	// Should be NSColor* color;

	styles = [NSMutableArray new];

	keywords = [NSArray arrayWithObjects:@"if", @"else", @"for", @"while", @"do", @"id", @"break", @"continue", @"nil", @"return", @"YES", @"NO", nil];
	style = [NSDictionary dictionaryWithObjects:
			[NSArray arrayWithObjects:@"ObjC", keywords, font, color, nil]
			    forKeys:[NSArray arrayWithObjects: @"name", @"keywords",
										@"font", @"color", nil]];
	[styles addObject:style];

	keywords = [NSArray arrayWithObjects:@"persistent", @"session", @"action", @"WOApp", nil];
	style = [NSDictionary dictionaryWithObjects:
		[NSArray arrayWithObjects:@"WebObjects", keywords, font, color, nil]
			    forKeys:[NSArray arrayWithObjects: @"name", @"keywords",
										@"font", @"color", nil]];
	[styles addObject:style];

	keywords = [NSArray arrayWithObjects:@"NSArray", @"NSMutableArray", @"NSDictionary", @"NSMutableDictionary", @"NSString", @"NSMutableString", @"NSNumber", @"NSDate", @"NSCalendarDate", @"@", nil];
	style = [NSDictionary dictionaryWithObjects:
		[NSArray arrayWithObjects:@"Foundation", keywords, font, color, nil]
			    forKeys:[NSArray arrayWithObjects: @"name", @"keywords",
										@"font", @"color", nil]];
	[styles addObject:style];
}

- (void)_applyStyles{
	NSEnumerator* enumerator = [styles objectEnumerator];
	NSDictionary* style;

	while (style = [enumerator nextObject]) {
		NSEnumerator* keys = [[style objectForKey:@"keywords"]
							 objectEnumerator];
		NSString* key;

		while (key = [keys nextObject]) {
			[scriptText _applyStyleTo:key font:[style objectForKey:@"font"]
						color:[style objectForKey:@"color"]];
		}
	}
}

- (void)_applyStyleTo:(NSString*)key font:font color:color{
}

- initFromDirectory:(const char*)aFilename
{
	NSString* path;
	NSString* dummyPath;

	[self setFilename:aFilename];
	//	dummyPath is like "Main.wo/Main" structure
	dummyPath = [dir stringByAppendingPathComponent:
		[[dir lastPathComponent] stringByDeletingPathExtension]];
	//	Now read dummyPath.html, dummyPath.wod, dummyPath.wos files
	path = [dummyPath stringByAppendingPathExtension:@"html"];
	template = [[NSMutableString alloc] initWithContentsOfFile:path];
	path = [dummyPath stringByAppendingPathExtension:@"wod"];
	declaration = [[NSMutableString alloc] initWithContentsOfFile:path];
	path = [dummyPath stringByAppendingPathExtension:@"wos"];
	script = [[NSMutableString alloc] initWithContentsOfFile:path];
	//	Now show everything in a window. Should be in windowWillOpen
	// [[goto:(int)pos end:(int)end mode:(int)mode
	[[templateText setSelToStart] replaceSel:[template cString]];
	[[declarationText setSelToStart] replaceSel:[declaration cString]];
	[[scriptText setSelToStart] replaceSel:[script cString]];

	if([template length] &&		// HTML only component
			!([declaration length] || [script length]))
		[self showTemplateOnly:self];

	[window setMiniwindowImage:[NXImage findImageNamed:"wo"]];
 	[window makeKeyAndOrderFront:self];

	return self;
}

- window {
	return window;
}

- setFilename:(const char*)aFilename
{
	if([dir length] > 0)	[dir autorelease];
	dir = [NSString stringWithCString:aFilename];
	[dir retain];
	[window setTitleAsFilename:[dir cString]];
	return self;
}

- (NSString*)filename
{
	return dir;
}

- save:sender
{
	NSString* path;
	NSString* dummyPath;

	if([dir length] <= 0)	return [self saveAs:sender];
	[window setTitle: "Saving..."];
 	mkdir([dir cString], 0777);
	dummyPath = [dir stringByAppendingPathComponent:
		[[dir lastPathComponent] stringByDeletingPathExtension]];

	path = [dummyPath stringByAppendingPathExtension:@"html"];
	[self _writeText:templateText to:path];
	template = [[NSMutableString alloc] initWithContentsOfFile:path];

	path = [dummyPath stringByAppendingPathExtension:@"wod"];
	[self _writeText:declarationText to:path];
	declaration = [[NSMutableString alloc] initWithContentsOfFile:path];

	path = [dummyPath stringByAppendingPathExtension:@"wos"];
	[self _writeText:scriptText to:path];
	script = [[NSMutableString alloc] initWithContentsOfFile:path];

	[window setTitleAsFilename:[dir cString]];
	[window setDocEdited:NO];

 return self;
}

- saveAs:sender
{
	id panel;
	char *file;

	if(![dir length]){
		dir = [NSString stringFromCString:NXHomeDirectory()];
		file = (char*)[window title];
		}
	else
	 	if([dir isEqualToString:[dir lastPathComponent]])
			file = (char*)[window title];
		else
			file = [[dir lastPathComponent] cString];
	panel = [SavePanel new];
	[panel setRequiredFileType:"wo"];
	if( [panel runModalForDirectory:[dir cString] file:file] ){
	 	[self setFilename:[panel filename]];
	 	return [self save:sender];
		}
	return nil;
}

- print:sender{
	[templateText printPSCode:sender];
	[declarationText printPSCode:self];
	[scriptText printPSCode:self];

	return self;
}

- (BOOL)shouldRunPrintPanel:aView{
	return NO;
}


- textDidChange:sender
{
	[window setDocEdited:YES];
	return self;	
}

- _writeText:aText to:(NSString*)path
{	// If the file does no exist...we will create it.
	// But if there is no real text for that file...we won't save anything !
	if([aText textLength]){	
	// Well the streams won't truncate a file so we have to do it on our own.
	// We should do a backup of existing files here if requested.
		FILE		* dummyStream;
		NXStream 	* aStream;
	
		dummyStream = fopen([path cString], "w");
		if(dummyStream) fclose(dummyStream);
		
		aStream = NXMapFile([path cString], NX_WRITEONLY);
		[aText writeText:aStream];
	
		NXSaveToFile(aStream, [path cString]);
		NXCloseMemory(aStream, NX_FREEBUFFER);
		}

	return self;
}

- _parseMethods
{
	return self;
}

- _parseElements
{
	return self;
}

- showAllFiles:sender
{
	NXRect	bounds;
	id		aView;

	[window disableDisplay];
	
	// Resize the declaration view to some useful none-zero value.
	// This is necessary to see anything at all.
	// We take the declaration because it will then end up as the smallest view.
	
	aView = [[declarationText superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :20];

	// Make the scriptText and templateText small enough..and redraw the window.
	
	aView = [[scriptText superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :50];
	aView = [[templateText superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :50];

	[window disableDisplay];
	[window display];
	return self;
}

- showTemplateOnly:sender
{
	[self _resizeCheatWindowFor:templateText
						andHide:declarationText :scriptText];

	return self;
}

- showDeclarationOnly:sender
{
	[self _resizeCheatWindowFor:declarationText
						andHide:templateText :scriptText];

	return self;
}

- showScriptOnly:sender
{
	[self _resizeCheatWindowFor:scriptText
						andHide:templateText :declarationText];

	return self;
}

- _resizeCheatWindowFor:mainView andHide:secondView :thirdView
{
	NXRect	bounds;
	id		aView;

	[window disableDisplay];
	
	// Resize the main view to some useful none-zero value.
	// This is necessary to see anything at all.
	
	aView = [[mainView superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :20];

	// Make the secondView and thirdView small enough..and redraw the window.
	
	aView = [[secondView superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :0];
	aView = [[thirdView superview] superview];
	[aView getBounds:&bounds];
	[aView sizeTo:bounds.size.width :0];

	[window disableDisplay];
	[window display];
	return self;
}

- windowWillClose:sender
{
	if( [sender isDocEdited] ){
		const char* fname;
		int q;
	
		fname = [dir cString] ? [dir cString] : [sender title];
		if( rindex(fname,'/') )	fname = rindex(fname,'/') + 1;
		q = NXRunAlertPanel("Save", "Save changes to %s?",
				"Save", "Don't save", "Cancel", fname);
		if( q == 1 )
			if( ![self save:nil] )	return nil;
		if( q == -1 )	return nil;	
		}
	[sender setDelegate: nil];
 	[self dealloc];
	return self;
}

- (void)dealloc
{

	[dir release];
	[template release];
	[declaration release];
	[script release];

}
@end
