/*
 *	BlatMailer.m
 *
 *	Objective-C wrapper class for BLAT under Windows NT
 *
 *	Copyright (C) 1996 Luke Howard lukeh@xedoc.com
 *
 *	You may freely copy, distribute and reuse the code in this example as long
 *	as you maintain the copyright notice.
 *
 *	Luke Howard disclaims any warranty of any kind, expressed or implied, as to
 *	its fitness for any particular use.
 *
 *	The following copyright notice is extracted from the BLAT documentation:
 *
 *	License to use Blat
 *	The authors of Blat have placed it in the public domain. This means you
 *	can use it free of charge for any purpose you like, with no conditions being
 *	placed on its use by us. The source code is also available free of charge and
 *	under the same conditions as the executables.

 *	
 *	You have permission to modify, redistribute, hoard, or even sell Blat in its
 *	executable or source form. If you do sell Blat, though, we'd appreciate it if
 *	you'd provide your own support (and send us a free copy).  We cannot take any
 *	support load for Blat (we've got better things to do). 
 *	
 *	Various bits of the source code are copyright by other people/organizations.
 *	Look in the source code for copyright ownership.
 *	
 *	The authors of the package are not responsible for any damage or losses that
 *	the usage of Blat may cause. We are especially not responsible for the misuse
 *	of the SMTP (or other) mail system.
 *
 */

#ifdef USE_WO_BLAT

#define BWO_VERSION "1.4"

#ifdef OPENSTEP
#import <Foundation/Foundation.h>
#else
#import <foundation/NSString.h>
#import <foundation/NSArray.h>
#import <foundation/NSDictionary.h>
#import <foundation/NSUtilities.h>
#import <foundation/NSScanner.h>
#import <foundation/NSValue.h>
#endif

#import "BlatMailer.h"
#import "BlatSupport.h"

#ifdef WIN32 // this really has to be defined!
#import <winnt-pdo.h>
#import <windows.h>
#import <malloc.h> // for alloca
#import <stdio.h>
#import <stdlib.h>
#import <iostream.h>
/* generic socket DLL support */
#import "gensock.h"

#define __far far
#define huge far
#define __near near
#endif               

#define NEWLINE "\r\n"

@interface BlatMailer (InternalMethods)
- (NSString *)parseHeaderLine:(NSString *)header;
- (NSString *)makeRecipientListFromHeaders:(NSDictionary *)headers;
@end

@implementation BlatMailer
+ (BlatMailer *)sharedInstance
{
	static BlatMailer *me = nil;
	
	if (me == nil)
		me = [[[[self class] alloc] init] autorelease];
	
	return me;
}

- init
{
	[super init];
	if (GetRegEntry() != 0)
		return nil;

	return self;
}

- (void)setServer:(NSString *)host
{
	smtpHost = [host retain];
}

- (void)dealloc
{
	[super dealloc];
	[smtpHost release];
}

- (void)setQuiet:(BOOL)val
{
	quiet=val;
}

- (BOOL)quiet
{
	return quiet;
}

- (int)blatMail:(NSString *)message withHeaders:(NSDictionary *)suppliedHeaders
{
// code based on Blat.C (aka BLAT.CPP)
	NSMutableString *buffer = [[[NSMutableString alloc] init] autorelease];
	NSMutableDictionary *headers;
	NSString *allRecipients;
	NSEnumerator *e;
	NSString *key;
	NSString *loginname;
	NSString *oldMailer;
	char tzString[64];
	
	SYSTEMTIME curtime;
	TIME_ZONE_INFORMATION tzinfo;
	char * days[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char * months[] = { "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
	DWORD retval;
	int retcode;
	int hours, minutes;

	if (suppliedHeaders)
		headers = [suppliedHeaders mutableCopy];
	else
		return -1;

	if (get_sender() != NULL)
		loginname = [NSString stringWithCString:get_sender()];
	else
		loginname = [headers objectForKey:@"From"];

	GetLocalTime( &curtime );
	retval = GetTimeZoneInformation( &tzinfo );

	hours = (int) tzinfo.Bias / 60;
	minutes = (int) tzinfo.Bias % 60;
 	if( retval == TIME_ZONE_ID_STANDARD ) 
	{
	 hours += (int) tzinfo.StandardBias / 60;
	 minutes += (int) tzinfo.StandardBias % 60;
	}
	else
  	{
	 hours += (int) tzinfo.DaylightBias / 60;
	 minutes += (int) tzinfo.DaylightBias % 60;
	}

	sprintf( tzString, "%+03d%02d", -hours, -minutes );
    // rfc1036 & rfc822 acceptable format
    // Mon, 29 Jun 94 02:15:23 GMT

	[headers setObject:[NSString stringWithFormat:@"%s, %.2d %s %.2d %.2d:%.2d:%.2d %s",
      days[curtime.wDayOfWeek],
      curtime.wDay,
      months[curtime.wMonth - 1],
      curtime.wYear,
      curtime.wHour,
      curtime.wMinute,
      curtime.wSecond, tzString] forKey:@"Date"];

//	[dateString appendString:[NSString stringWithCString:tzString]];
//	[headers setObject:dateString forKey:@"Date"];

	[headers setObject:loginname forKey:@"Sender"];

	allRecipients = [[self makeRecipientListFromHeaders:headers] retain];
	
	if ([headers objectForKey:@"X-Mailer"] != nil) {
		oldMailer = [headers objectForKey:@"X-Mailer"];
		[headers setObject:[NSString stringWithFormat:@"%@ via BlatMailer WO %s", oldMailer, BWO_VERSION] 
			forKey:@"X-Mailer"];
	} else {
		[headers setObject:[NSString stringWithFormat:@"BlatMailer WO %s",BWO_VERSION] forKey:@"X-Mailer"];
	}
	
	if ([headers objectForKey:@"MIME-Version"] != nil)
	{
	// fix mime content
		NSLog(@"Mime not supported properly yet.");
	} 

	// enumerate headers and add to buffer
	e = [headers keyEnumerator];
	while(key = [e nextObject]) {
		[buffer appendString:[NSString stringWithFormat:@"%@: %@%s", key,
			[headers objectForKey:key],
			NEWLINE]];
	}

	// separating newline	
	[buffer appendString:[NSString stringWithCString:NEWLINE]];
	// append message
	[buffer appendString:message];

#ifdef DEBUG
	NSLog(@"Loginname = %@", loginname);
	NSLog(@"allRecipients = %@", allRecipients);
	NSLog(@"\n%@\n", buffer);
#endif

	retcode = prepare_smtp_message( [loginname cString], [allRecipients cString] );
	if( !retcode )
	{
		retcode = send_smtp_edit_data([buffer cString] ); 
		if( !retcode )
			finish_smtp_message();
   		close_smtp_socket();
	}
	
	return retcode;
}

- (NSString *)makeRecipientListFromHeaders:(NSDictionary *)headers
{
	NSMutableString *string = [[[NSMutableString alloc] init] autorelease];
	
	[string appendString:[self parseHeaderLine:[headers objectForKey:@"To"]]];
	
	if ([headers objectForKey:@"Cc"] != nil)
	{
		[string appendString:@","];
		[string appendString:[self parseHeaderLine:[headers objectForKey:@"Cc"]]];
	}

	if ([headers objectForKey:@"Bcc"] != nil)
	{
		[string appendString:@","];
		[string appendString:[self parseHeaderLine:[headers objectForKey:@"Bcc"]]];	
	}
	
	return string;
}

- (NSString *)descriptionForErrorCode:(int)code
{
// incomplete
	NSString *str;
	
	switch (code)
	{
		case 0:
			str = [NSString stringWithFormat:@"OK"];
		default:
			str = [NSString stringWithFormat:@"Unknown error %d", code];
	}

	return str;
}

- (NSString *)parseHeaderLine:(NSString *)header
{
//
//	We do two passes: one to get rid of whitespace and full names within
//	brackets, and then another to get rid of characters not in <> within
//	an address.
//
//	NSCharacterSet *skipChars = [NSCharacterSet characterSetWithCharactersInString:@" \n"];
	NSCharacterSet *angle_openers = [NSCharacterSet characterSetWithCharactersInString:@"<"];
	NSCharacterSet *angle_closers = [NSCharacterSet characterSetWithCharactersInString:@">"];
	NSCharacterSet *round_openers = [NSCharacterSet characterSetWithCharactersInString:@"("];
	NSCharacterSet *round_closers = [NSCharacterSet characterSetWithCharactersInString:@")"];

//	get recipients into an array
	NSMutableArray *allRecipients = [[[NSMutableArray alloc]
					init] autorelease];

	unsigned int i = 0, count;

	[allRecipients addObjectsFromArray:[header componentsSeparatedByString:@","]];
	count = [allRecipients count];

	for(i=0; i<count; i++)
	{
		NSScanner *theScanner;
		BOOL done = NO;

		theScanner = [NSScanner scannerWithString:[allRecipients objectAtIndex:i]];
//		[theScanner setCharactersToBeSkipped:skipChars];

		while([theScanner isAtEnd] == NO && done == NO)
		{
			BOOL scanResult;
			NSString *tmp;

			scanResult = [theScanner scanUpToCharactersFromSet:angle_openers
				intoString:NULL];

			if (scanResult == NO) {
				break;
			}

			scanResult = [theScanner scanCharactersFromSet:angle_openers
				intoString:NULL];

			if (scanResult == NO) {
				break;
			}

			scanResult = [theScanner scanUpToCharactersFromSet:angle_closers
				intoString:&tmp];

			if (scanResult == NO) {
				break;
			}

			[allRecipients replaceObjectAtIndex:i withObject:tmp];
			done = YES;
		}
		
		[theScanner setScanLocation:0];

		while([theScanner isAtEnd] == NO && done == NO)
		{
			BOOL scanResult;
			NSString *tmp;

			scanResult = [theScanner scanUpToCharactersFromSet:round_openers
				intoString:&tmp];

			if (scanResult == NO) {
				break;
			}

			[allRecipients replaceObjectAtIndex:i withObject:tmp];
			done = YES;
		}
			
	}
	return [allRecipients componentsJoinedByString:@","];
}
@end

void ns_log(char *msg)
{
	if (msg != NULL)
		NSLog(@"%@", [NSString stringWithCString:msg]);
}

#endif
