/*
 *	Sendmail.m
 *
 *	Copyright (C) 1996 Luke Howard <lukeh@xedoc.com>
 *
 *	You may freely copy, distribute and reuse the code in this example for
 *	non-commercial purposes, otherwise you should contact me.
 *
 *	Luke Howard disclaims any warranty of any kind, expressed or implied, as to
 *	its fitness for any particular use.
 */
 
#ifdef OPENSTEP
#import <Foundation/Foundation.h>
#else
#import <foundation/NSCharacterSet.h>
#import <foundation/NSString.h>
#import <foundation/NSArray.h>
#import <foundation/NSDictionary.h>
#import <foundation/NSUtilities.h>
#import <foundation/NSScanner.h>
#import <foundation/NSValue.h>
#endif

#define SM_SIGNATURE @"WO-Sendmail 1.4"

// Tested under NT with BLAT
#ifdef WIN32
#import <winnt-pdo.h>
#endif

#import <stdio.h>

#import "Sendmail.h"

#ifdef USE_WO_BLAT
#import "Blat/BlatMailer.h"
#else
#if defined(BLAT) && defined(WIN32)
#define PATH_TO_SENDMAIL @"\\blat\\blat.exe"
#else
 #ifdef WIN32
 #define PATH_TO_SENDMAIL @"\\sendmail\\sendmail.exe"
 #else
 #define PATH_TO_SENDMAIL @"/usr/lib/sendmail"
 #endif
#endif
#ifndef PATH_TO_SENDMAIL
#error Please specify PATH_TO_SENDMAIL in Sendmail.m.
#endif
#endif /* USE_WO_BLAT */

#ifdef WIN32
#define POPEN _popen
#define PCLOSE _pclose
#else
#define POPEN popen
#define PCLOSE pclose
#endif

@implementation Sendmail
+ (Sendmail *)sharedInstance
{
// thanks to seanw@xedoc.com
	static Sendmail *me = nil;

// this doesn't work from within WebScript
	if (me == nil) {
		me = [[[[self class] alloc] init] autorelease];
	}
	
	return me;
}

#if defined(USE_WO_BLAT) && defined(DEBUG)
- sharedBlat
{
	return sharedBlat;
}
#endif

- init
{
	[super init];
	mime=NO;
#ifdef USE_WO_BLAT
	sharedBlat = [[BlatMailer sharedInstance] retain];
#endif
	lastError = [[NSMutableString alloc] init];
	return self;
}

- (void)dealloc
{
	[lastError release];
	[super dealloc];
}

- (BOOL)postMessage:(NSString *)message 
	from:(NSString *)sender
	to:(NSArray *)recipient
	withSubject:(NSString *)subject
	andHeaders:(NSDictionary *)headers
{
	NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];
	NSString *recipients = [recipient componentsJoinedByString:@","]; 
	
	if (headers) {
		[dict addEntriesFromDictionary:headers];
	}

// NSDictionary sends the object a retain message	
	[dict setObject:subject forKey:@"Subject"];
	[dict setObject:sender forKey:@"From"];
	[dict setObject:recipients forKey:@"To"];
	[dict setObject:SM_SIGNATURE forKey:@"X-Mailer"];

	if (mime) {
		[dict setObject:@"1.0" forKey:@"MIME-Version"];
		[dict setObject:@"text/plain; charset=ISO-8859-1" forKey:@"Content-type"];
		[dict setObject:@"quoted-printable" forKey:@"Content-Transfer-Encoding"];
	}
	
	return [self postMessage:message withHeaders:dict];
}

- (void)setMime:(BOOL)val
{
	mime=val;
}

- (BOOL)mime
{
	return mime;
}

- (BOOL)postMessage:(NSString *)message
	withHeaders:(NSDictionary *)headers
#ifdef USE_WO_BLAT
{
	exitCode = [sharedBlat blatMail:message withHeaders:headers];
	
	[lastError setString:[sharedBlat descriptionForErrorCode:exitCode]];
	
	if (exitCode == 0)
		return YES;
		
	return NO;
}
#else
{
//
// we should use NSFileHandle here
//
// eg. NSFileHandle *sendmail =
//		[[NSFileHandle alloc] initWithNativeHandle:(void *)_popen([cmdLine cString], "w")]
// may work although it's no more portable. The "proper" way would be to fork() and exec()
// sendmail, and then redirect stdin using
//		NSFileHandle *sendmail = [NSFileHandle fileHandleWithStandardInput];
// which would eliminate the ugly shell escaping code. This isn't much use now anyway
// given NSFileHandle probably isn't in D'OLE 3.5/WOPro.
//
	FILE *sendmail;

#ifndef BLAT // only needed to enumerate headers
	NSString *key;
	NSEnumerator *e;
#endif

#ifdef WIN32 // don't need to escape?
	NSString *recipients = [headers objectForKey:@"To"];
#else
	NSString *recipients = [self escapeString:[headers objectForKey:@"To"]];
#endif
	NSMutableString *cmdLine = [[[NSMutableString alloc] initWithString:PATH_TO_SENDMAIL] autorelease];

	if (headers == nil) {
		[lastError setString:@"No headers supplied"];
		return NO;
	} else {
		[headers retain];
	}

#ifdef BLAT // BLAT isn't entirely like sendmail. Note header keys are case-sensitive here.
	[cmdLine appendString:[NSString stringWithFormat:@" - -t \"%@\"", recipients]];

	if ([headers objectForKey:@"Subject"] != nil) {
		[cmdLine appendString:[NSString stringWithFormat:@" -s \"%@\"",
			[headers objectForKey:@"Subject"]]];
	}

	if ([headers objectForKey:@"From"] != nil) {
		[cmdLine appendString:[NSString stringWithFormat:@" -i \"%@\"",
			[headers objectForKey:@"From"]]];
	}

	if ([headers objectForKey:@"Cc"] != nil) {
		[cmdLine appendString:[NSString stringWithFormat:@" -c \"%@\"",
			[headers objectForKey:@"Cc"]]];
	}

	if ([headers objectForKey:@"Bcc"] != nil) {
		[cmdLine appendString:[NSString stringWithFormat:@" -b \"%@\"",
			[headers objectForKey:@"Bcc"]]];
	}

#ifndef DEBUG
	[cmdLine appendString:@" -q"]; // turn on quiet mode for BLAT
#endif
#else
	[cmdLine appendString:[NSString stringWithFormat:@" -t \"%@\"", recipients]];
#endif

#ifdef DEBUG
	NSLog(@"about to start %@", cmdLine);
#endif
	sendmail = POPEN([cmdLine cString], "w");

	if (sendmail == NULL) {
		[lastError setString:@"Failed to popen() sendmail"];
		return NO;
	}

#ifndef BLAT // headers are specified on the command line
	e = [headers keyEnumerator];
	while(key = [e nextObject]) {
		fprintf(sendmail, "%s: %s\n", [key cString], [[headers objectForKey: key] cString]);
	}
	fprintf(sendmail, "\n"); // to separate headers
#endif
	
	fprintf(sendmail, "%s", [message cString]);

	exitCode = PCLOSE(sendmail);
	
	if (exitCode == 0) {
		[lastError setString:@"OK"];
		return YES;
	}
	
	[lastError setString:[NSString stringWithFormat:@"%@ returned %d", PATH_TO_SENDMAIL, exitCode]];
	return NO;
}
#endif

- (NSString *)lastError
{
	return lastError;
}

- (NSNumber *)lastExitCode
{
	return [NSNumber numberWithInt:exitCode];
}

- (NSString *)escapeString:(NSString *)str
{
// could be more efficiently implemented
	NSCharacterSet *escapedChars;
	NSScanner *theScanner;
	NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
		
	theScanner = [NSScanner scannerWithString:str];
	escapedChars = [NSCharacterSet characterSetWithCharactersInString:@"\"$;|&'`[]\\"];

	while ([theScanner isAtEnd] == NO)
	{
		BOOL scanResult;
		NSString *s1, *s2;
		
		scanResult = [theScanner scanUpToCharactersFromSet:escapedChars intoString:&s1];
		if (scanResult == NO)
			break;
		
		[result appendString:s1];
		
		[s1 release];
		
		scanResult = [theScanner scanCharactersFromSet:escapedChars intoString:&s2];
		if (scanResult == NO)
			break;
		
		[result appendString:@"\\"];
		[result appendString:s2];
		
		[s2 release];
	}
	
// there must be a better way
	[result appendString:[str substringFromIndex:[theScanner scanLocation]]];

	return result;
}

- (BOOL)postMessage:(NSString *)message
	from:(NSString *)sender
	to:(NSString *)aRecipient
	withSubject:(NSString *)subject
{
	return [self postMessage:message from:sender to:[NSArray arrayWithObjects:aRecipient,nil] withSubject:subject andHeaders:nil];
}

@end